DAT programName         byte "EncoderDemo_131212g", 0
CON
{{
  ******* Public Notes *******

  Author: Duane Degn
  Date First Released: December 10, 2013
  Date Latest Released: December 12, 2013

  MIT License

  The current object being demonstated has
  a resolution of 6us (at 80MHz). The main PASM
  loop takes 480 clock cycles to complete.
  It may be possible to increase this resolution
  if only two motors/encoders are used and
  to code controlling and reading the unused
  motors and enocoders is removed. 
  
  While there is motor control features in
  the object being demonstrated, this demo
  program doesn't not include a way to interface
  with the motor control aspects of the driver.
  
-------------------------------------------------  
  Motor Control Information:
-------------------------------------------------  
  The "FourQuadratureMotor" object can control
  motors requiring normal PWM input as well
  as continuous rotation servos.

  I have not tested the precision of the servo
  control signal. I doubt it has 1us resolution
  as the Servo32vX object does. If you have
  cogs available, you may wish to use a
  different object than "FourQuadratureMotor"
  for continuous rotation servo control.
  
  The program can not currently use both normal
  h-bridge motors and continuous rotation
  simultaneously. All motors are assumed to be 
  the same type. The program can use both
  types of direction control features. A
  h-bridge with single pin direction control
  may be used together with a dual pin control
  h-bridge.

  Set WHEEL_ENABLE_x to -1 if no motor used on
  that channel.
  
  If WHEEL_ENABLE_x if set to a valid I/O pin
  and WHEEL_RED_x is set to -1, the motor will
  be treated as a continuous rotation servo.

  If WHEEL_ENABLE_x and WHEEL_RED_x are both
  set to valid I/O pins and WHEEL_BLACK_x is
  set to -1, the WHEEL_RED_x pin will be used
  to set the direction of the motor. High for
  forward, low for reverse. Your h-bridge must
  support this type of interface in order to
  use it. This interface does not allow
  dynamic braking.
  
  If WHEEL_ENABLE_x, WHEEL_RED_x and
  WHEEL_BLACK_x are all set to valid I/O pins,
  WHEEL_RED_x and WHEEL_BLACK_x pins will be
  used to set the direction of the motor.
  Forward speeds will set WHEEL_RED_x high
  and WHEEL_BLACK_x low. The high and low
  states are switched for reverse.
  When speed is set to zero both direction
  pins will be set low.

-------------------------------------------------  
  Encoder Information:
-------------------------------------------------  
  The algorithm used in the encoder object
  comes out of JonnyMac's Spin Zone article
  "Spinning it Up with Encoders".

  Up to four quadrature encoders may be used
  with the "FourQuadratureMotor" object. The
  "A" and "B" sensor pins need to be
  consecutive I/O pins. Only the first of the
  two pins is listed in the "FIRST_ENCODER_x"
  section of the header object. Provisions
  have been made to reverse the direction
  of the encoder feedback if desired without
  the need to physically swap the wires
  connected to the Propeller. See comments
  within the program about setting the
  encoder direction.

  If an encoder is not used the "FIRST_ENCODER_x"
  pin should be set to -1.

  If an encoder pin is set to a valid I/O pin
  but the I/O pin is left to float, the input
  pins may register phantom transitions.

  There are several constants in the header
  object used to compute speeds.

  The constant "MAX_TRANSITIONS_PER_SEC" is
  used to scale the speed in order to have
  the measured speed have the same range as
  the PWM resolution. For example, if the
  "DEFALUT_PWM_RESOLUTION" constant in the
  child object is set to 1000, then a speed
  setting of 1000 should produce a measured
  speed of 1000. In order for the measured
  speed to match the input speed the constant
  "MAX_TRANSITIONS_PER_SEC" needs to be set
  to the appropriate value.
  
  This appropriate value may be calculated
  based on the motors known max rpm and
  transitions per revolution data or it
  may be directly measured by running this
  program while the motor runs at full
  speed. Currently, the motor will need to
  be powered from another source since this
  program does not yet have a way to input
  speed. 

  Rather than having a single constant for
  the number of transitions per revolution
  of the motor, I used two.
  "TRANS_PER_REV_NUMERATOR" and
  "TRANS_PER_REV_DENOMINATOR".
  This allows for non-integer encoder values.
  For example the Rover 5 has encoders which
  register 1000 transitions every 3 rotations
  of the wheel. Expressing the encoder
  resolution as a fraction allows this 1000/3
  resolution to be accurately stated with
  integers. 

  There are two direction arrays,
  "encoderDirection" and "directionCorrection".
  A pointer to the "encoderDirection" array
  is passed to the encoder object which
  populates the array with ones or negative
  ones based on the sensed direction the
  encoders are turning. The other array
  "directionCorrection" is filled with
  constants from the header object. These
  constants are used to correct the direction
  reported by the encoders so the wires
  of the encoders do not need to be physically
  swapped.
  
  
-------------------------------------------------  
  Features To Add:
-------------------------------------------------  
  Be able to input target speeds.

  Think about 64-bit math.
 
}}
{
  ******* Private Notes *******

  131210b "RPM_SPEED_MULTIPLIER" no longer need
  now that "integerMathEnhancer[0]" is being used.
  "RPM_SPEED_MULTIPLIER" now causes 32-bit overflow.
  10c Is working better but only a tenth of an
  rpm unit of accuracy.
  10g Try to maximize resolution of speed.
  10g Some sort of timing problem. It's not
  registering stopped consistently.
  10h Seems to work okay.
  10i Add divide with rounding method.
  10i The divide with rounding didn't change the
  values much. If there was a difference it was
  only in the very last digit. I don't think
  rounding is worth the time delay.
  10j Remove rounding method and clean up
  formatting a bit.
  10j Spell check comments.
  10k Simplify a few constants.
  11b Fix direction problem
  11b Break the debug process into multiple
  methods.
  12f Added speed input seems to work.
  Added a bunch of display code.
  
}
CON                     
                     
  _clkmode = xtal1 + pll16x 
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MICROSECOND = CLK_FREQ / 1_000_000

  TEMP_MESSAGE_DISPLAY_TIME = CLK_FREQ * 4
  
  MAX_POSSIBLE_ENCODERS = Encoders#TOTAL_ENCODERS
  ARRAYS_TO_FREEZE = 3
  LONGS_TO_FREEZE = MAX_POSSIBLE_ENCODERS * ARRAYS_TO_FREEZE
  MAX_POSITIVE = $7FFFFFFF
  LARGEST_PRACTICAL_MULTIPLIER = MAX_POSITIVE 
  LARGEST_POWER_OF_TEN_LIKELY = 10000

  HEADING_ROWS = 5
  MENU_LOC_Y = HEADING_ROWS
  MENU_ROWS = 10
  CYCLE_RESOLUTION_LOC_Y = MENU_LOC_Y + MENU_ROWS
  DATA_ROWS = 2
  ENCODER_DATA_LOC_Y = CYCLE_RESOLUTION_LOC_Y + DATA_ROWS 
  ENCODER_DATUM_ROWS = 4
  ENCODER_DATA_ROWS = ENCODER_DATUM_ROWS * Encoders#TOTAL_ENCODERS
  IO_PIN_STATES_LOC_Y = ENCODER_DATA_LOC_Y + ENCODER_DATA_ROWS + 1
  IO_PIN_STATES__ROWS = 5
  TARGET_SPEED_LOC_Y = IO_PIN_STATES_LOC_Y + IO_PIN_STATES__ROWS
  TARGET_SPEED_ROWS = 5
  SPEED_INPUT_LOC_Y = TARGET_SPEED_LOC_Y + TARGET_SPEED_ROWS 
  SPEED_INPUT_ROWS = 4
  TEMP_MESSAGE_LOC_Y = SPEED_INPUT_LOC_Y + SPEED_INPUT_ROWS
  TEMP_MESSAGE_ROWS = 3
  
  QUOTE = 34  ' double quotes
  LINE_SIZE = 40
  ASTRIX_IN_ROW = LINE_SIZE
  ' activeParameter enumeration
  #0, PROPORTIONAL, INTEGRAL, DERIVATIVE, ACCELERATION, {
     DELTA_TIME, TOP_SPEED_FEEDBACK,} FREQUENCY, RESOLUTION

  TOTAL_PARENT_PARAMETERS = ACCELERATION + 1 'TOP_SPEED_FEEDBACK + 1

  #0, ERROR_MESSAGE, SPEED_INPUT_MESSAGE
  
  MAX_INDEX_TEMP_MESSAGE_TYPES = SPEED_INPUT_MESSAGE 
  TEMP_MESSAGE_TYPES = MAX_INDEX_TEMP_MESSAGE_TYPES + 1  
  
VAR

  long directionCorrection[MAX_POSSIBLE_ENCODERS], output[MAX_POSSIBLE_ENCODERS]
  long targetSpeed[MAX_POSSIBLE_ENCODERS]
  long zeroSpeedTime
  long newCount[MAX_POSSIBLE_ENCODERS], pulseToSpeedNumerator
  long transPerSecSpeed[MAX_POSSIBLE_ENCODERS], scaledSpeed[MAX_POSSIBLE_ENCODERS] 
  long rpmSpeed[MAX_POSSIBLE_ENCODERS]', linearSpeed[MAX_POSSIBLE_ENCODERS], linearUnitsPtr 
  long integerMathEnhancer[2], otherPortionOfEnhancer[2], powerOfTenPortionOfEnhancer[2]
  long powerOfTenPortionsOfEnhancer, otherPortionsOfEnhancer
  long parentParameter[TOTAL_PARENT_PARAMETERS], tempMessageTimer[TEMP_MESSAGE_TYPES]
  
  '' The following longs need to be kept in order and together.
  long frozenTransitionTime[MAX_POSSIBLE_ENCODERS], frozenLastPulse[MAX_POSSIBLE_ENCODERS]
  long frozenEncoderCount[MAX_POSSIBLE_ENCODERS]
  long previousTransitionTime[MAX_POSSIBLE_ENCODERS], previousLastPulse[MAX_POSSIBLE_ENCODERS]
  long previousEncoderCount[MAX_POSSIBLE_ENCODERS]
  long legacyLongsNotUsed[MAX_POSSIBLE_ENCODERS]
  long transitionTime[MAX_POSSIBLE_ENCODERS], timeOfLastPulse[MAX_POSSIBLE_ENCODERS]
  long totalEncoderCount[MAX_POSSIBLE_ENCODERS]
  long encoderDirection[MAX_POSSIBLE_ENCODERS] 
  '' The above longs need to be kept in order and together.

  byte activeChannelFlag[MAX_POSSIBLE_ENCODERS], stillStoppedFlag[MAX_POSSIBLE_ENCODERS]
  byte activeParameter, tempMessageFlag[TEMP_MESSAGE_TYPES], tempMessageLoc[2 * TEMP_MESSAGE_TYPES]
  
OBJ

  Pst : "Parallax Serial Terminal"                      ' uses one cog
  Encoders : "FourQuadratureMotors131212b"              ' uses one cog
  'Header : "HeaderDemo_131211a"
  Header : "HeaderVexTreads131212a"
  
PUB Main 
'' This method starts in cog #0
'' This method starts two additional cogs (encoder/motor and serial driver).
'' Presently there are five unused cogs.

  directionCorrection[0] := Header#FORWARD0  ' Note about direction variables
  directionCorrection[1] := Header#FORWARD1  ' in comments under "Encoder Information."
  directionCorrection[2] := Header#FORWARD2
  directionCorrection[3] := Header#FORWARD3
  '' "directionCorrection" allows direction reversal without requiring wires be physically moved.
  '' If the encoder direction is reversed from direction desired, change the "FORWARDx" to -1 
  '' in header object.
  '' Motor direction may be reversed by swapping "RED" and "BLACK" pin asignments in header
  '' object.
   
  'linearUnitsPtr := Header.GetLinearUnits
  
  pulseToSpeedNumerator := (clkfreq / Header#MAX_TRANSITIONS_PER_SEC) * Encoders#DEFALUT_PWM_RESOLUTION
  '' pulseToSpeedNumerator used to scale speed in attempt to get target speed and measured speed
  '' to match

  ComputerMathEnhancers
    
  zeroSpeedTime := pulseToSpeedNumerator + (pulseToSpeedNumerator / 2)
  '' How long a pulse needs to be to be considered stopped.
  
  Pst.Start(115200)
  
  Encoders.StartEncoders(Header.GetEncoderPins, @totalEncoderCount, @transitionTime, Header.GetEnablePins, {
  } Header.GetRedPins, Header.GetBlackPins, @output, @encoderDirection)

  'waitcnt(clkfreq * 2 + cnt)

  RefreshStaticScreen
  
  DebugLoop
    
  'refreshInterval := clkfreq / Header#REFRESH_RATE
  'nextRefreshTime := cnt + refreshInterval
PUB DebugLoop | frozenCnt, frozenIna, combinedDirection 
  
  repeat
    repeat result from 0 to MAX_INDEX_TEMP_MESSAGE_TYPES
      if tempMessageFlag[result]
        if cnt - tempMessageTimer[result] > TEMP_MESSAGE_DISPLAY_TIME
          tempMessageFlag[result] := 0
          ClearLines(tempMessageLoc[2 * result], tempMessageLoc[(2 * result) + 1])
  
    result := Pst.RxCount
    if result
      ReadInput
    
    DisplayResolution

    longmove(@previousTransitionTime, @frozenTransitionTime, LONGS_TO_FREEZE)
    longmove(@frozenTransitionTime, @transitionTime, LONGS_TO_FREEZE)
    '' Encoder data is copied to "frozen" arrays so the values don't change
    '' while being compared. This insures the "previous" value stored is
    '' the same value used in the comparison.
    
    frozenCnt := cnt

    CheckForStoppedChannels    
    'DisplayEnhancers
    Encoders.RefreshSpeed
    DisplayTargetSpeeds
    
    repeat result from 0 to Encoders#MAX_ENCODER_INDEX
      combinedDirection := encoderDirection[result]  * directionCorrection[result]
      
      if frozenCnt - frozenLastPulse[result] < zeroSpeedTime and stillStoppedFlag[result] == 0
        transPerSecSpeed[result] := ComputeTransPerSec(frozenTransitionTime[result], combinedDirection)
        rpmSpeed[result] := ComputeRpm(frozenTransitionTime[result], combinedDirection)
        scaledSpeed[result] := ComputeScaledSpeed(frozenTransitionTime[result], combinedDirection)
      else ' slower than allowed
        transPerSecSpeed[result] := 0
        rpmSpeed[result] := 0
        scaledSpeed[result] := 0
      ifnot activeChannelFlag[result]  ' Channel not currently active
        if timeOfLastPulse[result]  ' if pulse has been written by encoder driver set channel active
          activeChannelFlag[result] := 1
        else
          next  ' Channel still not active skip display portion of loop
      DisplayEncoderChannel(result)
      
      {StepByStepDebug(result)
      Pst.Str(string(13, "encoderDirection[result] = "))
      Pst.Dec(encoderDirection[result])
      Pst.Str(string(13, "directionCorrection[result] = "))
      Pst.Dec(directionCorrection[result])
      Pst.Str(string(13, "combinedDirection = "))
      Pst.Dec(combinedDirection)   }
      
    DividingLine      
    DisplayIo
    
    waitcnt(clkfreq / 10 + cnt) 

PUB ReadInput : localCharacter

  localCharacter := Pst.CharIn
  
  case localCharacter
    "0".."0" + Encoders#MAX_ENCODER_INDEX:
      GetSpeedInput(localCharacter)
    "c", "C":
      RefreshStaticScreen

    "l", "L":
    "p", "P", "i", "I", "d", "D", "a", "A", "f", "F", "r", "R":
      ActivateParameter(localCharacter)
    other:
      DisplayError(@characterInputError)
        
PUB ActivateParameter(localParameter)

  'result := activeParameter
  
  case localParameter
    {"p", "P":
      activeParameter := PROPORTIONAL   
    "i", "I":
      activeParameter := INTEGRAL   
    "d", "D":
      activeParameter := DERIVATIVE   
    "a", "A":
      activeParameter := ACCELERATION   
    "m", "M":
      activeParameter := DELTA_TIME   
    "t", "T":
      activeParameter := TOP_SPEED_FEEDBACK   }
    "f", "F":
      activeParameter := FREQUENCY 
    "r", "R":
      activeParameter := RESOLUTION   
    other:
      DisplayError(@parameterNotYetError)
    
  'if activeParameter <> result
  
PUB GetSpeedInput(channelIndex) | localResolution

  channelIndex -= "0"           ' Convert from ASCII character to number
  localResolution := Encoders.GetResolution
  Pst.Position(0, SPEED_INPUT_LOC_Y)
  Pst.Str(string(" ****** Enter Speed For Motor #"))
  Pst.Dec(channelIndex)
  Pst.Str(string(" ****** "))
  Pst.ClearEnd
  Pst.NewLine   ' +1
  Pst.ClearEnd
  Pst.NewLine   ' +2
                 '01234567890123
  Pst.Str(string(" targetSpeed["))
  Pst.Dec(channelIndex)
                 '45678
  Pst.Str(string("] = "))
  Pst.ClearEnd
  Pst.Position(18, SPEED_INPUT_LOC_Y + 2)
  result := Pst.DecIn
  if ||result > localResolution
    Pst.Position(18, SPEED_INPUT_LOC_Y + 1)
    Pst.Str(string(" The value ", QUOTE))
    Pst.dec(result)
    Pst.Str(string(QUOTE, " is out of range. Valid speeds are between -"))
    Pst.dec(localResolution)
    Pst.Str(string(" and +"))
    Pst.dec(localResolution)
    Pst.Char(".")
    Pst.ClearEnd
    Pst.Str(string(13, " The speed setting of motor #"))
    Pst.dec(channelIndex)
    Pst.Str(string(" has not been changed."))
    Pst.ClearEnd
    DisplayError(@speedInputError)
  else
    output[channelIndex] := targetSpeed[channelIndex] := result
    Pst.Position(18, SPEED_INPUT_LOC_Y + 2)
    Pst.dec(targetSpeed[channelIndex])  
    Pst.ClearEnd
    Pst.Position(18, SPEED_INPUT_LOC_Y + 1)
    Pst.Str(string(" New speeds accepted."))
    
  tempMessageTimer[SPEED_INPUT_MESSAGE] := cnt
  tempMessageFlag[SPEED_INPUT_MESSAGE] := 1 

  tempMessageLoc[2 * SPEED_INPUT_MESSAGE] := SPEED_INPUT_LOC_Y
  tempMessageLoc[(2 * SPEED_INPUT_MESSAGE) + 1] := SPEED_INPUT_LOC_Y + 2
                                
PUB DisplayResolution

  Pst.Position(0, CYCLE_RESOLUTION_LOC_Y)
  Pst.Str(string("  PWM and Encoder resolution = "))
  result := EncoderS.GetDebugTime
  Pst.Dec(result)
  Pst.Str(string(" clock cycles or ")) 
  Pst.Dec(result / MICROSECOND)
  Pst.Str(string(" us")) 
  Pst.ClearEnd
  DividingLine
    
PUB DisplayTargetSpeeds

  Pst.Position(0, TARGET_SPEED_LOC_Y)
  
  repeat result from 0 to Encoders#MAX_ENCODER_INDEX
    DisplayTargetSpeed(result)
    if result <> Encoders#MAX_ENCODER_INDEX  
      Pst.NewLine
  DividingLine
    
PUB DisplayTargetSpeed(channel)

  Pst.Str(string(" targetSpeed["))
  Pst.Dec(channel)
  Pst.Str(string("] = "))
  Pst.Dec(targetSpeed[channel]) 
  Pst.ClearEnd
    
PUB DisplayIo | frozenIna

  Pst.Position(0, IO_PIN_STATES_LOC_Y)
  Pst.str(string("             I/O Pin Numbers"))
  Pst.ClearEnd
  Pst.str(string(13, "      tens:   3           2            1"))
  Pst.ClearEnd
  Pst.str(string(13, "     units:  1098_7654_3210_9876_5432_1098_7654_3210"))
  Pst.ClearEnd
  frozenIna := ina
  Pst.str(string(13, " ina state: "))
  ReadableBin(frozenIna)
  Pst.ClearEnd
  DividingLine
    
PUB DisplayEncoderChannel(channelIndex)

  {newCount[channelIndex] := frozenEncoderCount[channelIndex] - previousEncoderCount[channelIndex] 
  Pst.NewLine
  Pst.ClearEnd
  Pst.Str(string(13, "       newCount["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  Pst.Dec(newCount[channelIndex])
  Pst.ClearEnd   }
  Pst.Position(0, ENCODER_DATA_LOC_Y + (channelIndex * ENCODER_DATUM_ROWS))
  Pst.Str(string("     totalCount["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(frozenEncoderCount[channelIndex])
  'Pst.ClearEnd
  Pad(frozenEncoderCount[channelIndex], 10)
  Pst.PositionX(31)
  Pst.Str(string(", transitionTime["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(frozenTransitionTime[channelIndex])
  'Pst.Str(string(", = "))
  Pad(frozenTransitionTime[channelIndex], 8)
  Pst.PositionX(62)
  Pst.Str(string(" clock cycles, average of "))
  Pst.Dec(Encoders#BUFFER_SIZE_ENCODER_TIME)
  Pst.Str(string(" readings"))
  Pst.ClearEnd
  Pst.Str(string(13, "          speed["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(transPerSecSpeed[channelIndex])
  Pad(transPerSecSpeed[channelIndex], 6)
  Pst.PositionX(28)
  'Pst.ClearEnd 
  Pst.Str(string("(transitions per second)"))
  Pst.ClearEnd
  'Pst.NewLine 
  'Pst.ClearEnd 
  Pst.Str(string(13, "    scaledSpeed["))
  Pst.Dec(channelIndex)
  Pst.Str(string("] = "))
  'Pst.Dec(scaledSpeed[channelIndex])
  'Pst.ClearEnd
  Pad(scaledSpeed[channelIndex], 6)
  Pst.PositionX(28)         
  Pst.Str(string(" Scaled so max transitions per sec ("))
  Pst.Dec(Header#MAX_TRANSITIONS_PER_SEC)
  Pst.Str(string(") equals full speed of "))
  Pst.Dec(Encoders.GetResolution) 
  Pst.Str(string(" (current PWM resolution)."))
  Pst.ClearEnd
  'Pst.NewLine
  'Pst.ClearEnd
  Pst.Str(string(13, " rpm = "))
  DecPoint(rpmSpeed[channelIndex], powerOfTenPortionsOfEnhancer)
  Pst.Str(string("      "))
  'Pst.ClearEnd
  Pst.Str(string(" Based on "))
  Pst.Dec(Header#TRANS_PER_REV_NUMERATOR) 
  if Header#TRANS_PER_REV_DENOMINATOR <> 1
    Pst.Str(string(" / "))
    Pst.Dec(Header#TRANS_PER_REV_DENOMINATOR)
  Pst.Str(string(" transitions per revolution."))
  
  Pst.ClearEnd
  'DividingLine
  
PUB RefreshStaticScreen

  Pst.Clear
  DisplayHeading
  DisplayMenu
  
PUB DisplayHeading

  Pst.Home
  Pst.ClearEnd
  Pst.Str(string(13, "        name of program = "))
  Pst.Str(@programName)
  Pst.ClearEnd
  Pst.Str(string(13, " name of encoder object = "))
  Pst.Str(Encoders.GetObjectName)
  Pst.ClearEnd
  Pst.Str(string(13, "  name of header object = "))
  Pst.Str(Header.GetObjectName)
  Pst.ClearEnd
  'Pst.NewLine
  DividingLine
  'Pst.ClearEnd

PUB DisplayMenu

  Pst.Position(0, MENU_LOC_Y)
  Pst.Str(string("   Menu"))
  Pst.ClearEnd
  Pst.Str(string(" To enter speed of a motor press the motor ID number 0 through "))
  Pst.Dec(Encoders#MAX_ENCODER_INDEX)
  Pst.ClearEnd
  Pst.NewLine
  Pst.ClearEnd
  Pst.Str(string(13, " Alternately Press One of the Following Keys"))
  Pst.ClearEnd
  Pst.Str(string(13, "  c) Clear screen and display header and menu."))
  Pst.ClearEnd
  {
  Pst.Str(string(13, "  l) change active parameter Large adjustment amount."))
  Pst.ClearEnd
  Pst.Str(string(13, "  p) set Proportional as active parameer."))                   
  Pst.ClearEnd
  Pst.Str(string(13, "  i) set Integral as active parameer."))                   
  Pst.ClearEnd
  Pst.Str(string(13, "  d) set Differential as active parameer."))                   
  Pst.ClearEnd
  Pst.Str(string(13, "  a) set Acceleration as active parameer."))                   
  Pst.ClearEnd
  
  Pst.Str(string(13, "  f) enter new pwm Frequency."))                   
  Pst.ClearEnd
  Pst.Str(string(13, "  r) enter new pwm Resolution."))                   
  Pst.ClearEnd
  
  Pst.Str(string(13, "  m) set delta tiMe as active parameer."))                   
  Pst.ClearEnd
  Pst.Str(string(13, "  t) enter expected Top speed (transitions per second)."))                   
  Pst.ClearEnd
  }
  Pst.NewLine
  Pst.ClearEnd
  DividingLine          
                   
PUB DisplayError(errorPtr) | fillSpaces

  result := strsize(errorPtr)
  fillSpaces := ((ASTRIX_IN_ROW - result) - 4) / 2
  result := fillSpaces + (((ASTRIX_IN_ROW - result) - 4) // 2)
  Pst.Position(0, TEMP_MESSAGE_LOC_Y)
  Pst.Str(@astrixRow)
  Pst.ClearEnd  
  Pst.Str(string(13, "**"))
  repeat fillSpaces
    Pst.Char(32)
  Pst.Str(errorPtr)                  
  repeat result
    Pst.Char(32)
  Pst.Str(string("**"))
  Pst.ClearEnd
  Pst.NewLine
  Pst.Str(@astrixRow)
  Pst.ClearEnd
  tempMessageTimer[ERROR_MESSAGE] := cnt
  tempMessageFlag[ERROR_MESSAGE] := 1 

  tempMessageLoc[2 * ERROR_MESSAGE] := TEMP_MESSAGE_LOC_Y  
  tempMessageLoc[(2 * ERROR_MESSAGE) + 1] := TEMP_MESSAGE_LOC_Y + TEMP_MESSAGE_ROWS

PUB DividingLine

  Pst.NewLine
  Pst.Str(@dashRow) 
  Pst.ClearEnd
  Pst.NewLine
  
PUB CheckForStoppedChannels
'' Measuring speed from pulse lengths
'' become meaningless when an encoder
'' has stopped moving.

  repeat result from 0 to 3
    ifnot activeChannelFlag[result]
      next
    'Pst.Str(string("  , ["))
    'Pst.Dec(result)
    'Pst.Str(string("] "))
    stillStoppedFlag[result] := 0
    if scaledSpeed[result] == 0
      if timeOfLastPulse[result] == previousLastPulse[result]
        'Pst.Str(string("still zero"))
        'frozenLastPulse[result] :=  (frozenCnt - zeroSpeedTime) '- 1
        ' I'm not sure if the stopped encoder is handled correctly.
        stillStoppedFlag[result] := 1
      else
        'Pst.Str(string("  new zero"))
    else
      'Pst.Str(string("  not zero"))    
  'Pst.ClearEnd

PUB ComputeTransPerSec(time, direction)

  result := clkfreq / time * direction

PUB ComputeRpm(time, direction)

  result := (((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0] * {
          } direction) / time) * 60) * integerMathEnhancer[1]) / {
          } Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer
       
PUB ComputeScaledSpeed(time, direction)

  result := (pulseToSpeedNumerator / frozenTransitionTime[result]) * direction
        
PUB ComputerMathEnhancers
'' This method is used to find variables to optimize the precision
'' of the integer math.
'' These values need to be computed for each set of encoders 
'' based on the encoder resolution and speed of the motor.
'' These variables are not required but do improve the accuracy
'' of the computed speeds.
'' I plan to investigate using 64-bit math in order
'' to further improve the accuracy of the speed calculations.

  integerMathEnhancer[0] := (LARGEST_PRACTICAL_MULTIPLIER / clkfreq) / Header#TRANS_PER_REV_DENOMINATOR
  '' The pulse number used in the speed equations can result in significant rounding
  '' errors do to integer math. This factor is used to reduce the rounding error by
  '' increasing the original value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[0] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[0] := integerMathEnhancer[0] / powerOfTenPortionOfEnhancer[0]
    if otherPortionOfEnhancer[0] == 0
      powerOfTenPortionOfEnhancer[0] /= 10
  while otherPortionOfEnhancer[0] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[0] == 0
    otherPortionOfEnhancer[0] := 1

  integerMathEnhancer[0] := otherPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[0]

  result := clkfreq / ((Header#MAX_TRANSITIONS_PER_SEC * 3) / 2) ' shortest possible transition time
  integerMathEnhancer[1] := result / 60 
  '' This factor is used to reduce the rounding error by
  '' increasing the intermediate value to near the point of 32-bit(signed) rollover.
  
  powerOfTenPortionOfEnhancer[1] := LARGEST_POWER_OF_TEN_LIKELY
  repeat
    otherPortionOfEnhancer[1] := integerMathEnhancer[1] / powerOfTenPortionOfEnhancer[1]
    if otherPortionOfEnhancer[1] == 0
      powerOfTenPortionOfEnhancer[1] /= 10
  while otherPortionOfEnhancer[1] == 0 and powerOfTenPortionOfEnhancer[0] > 1
  if otherPortionOfEnhancer[1] == 0
    otherPortionOfEnhancer[1] := 1
  
  integerMathEnhancer[1] := otherPortionOfEnhancer[1] * powerOfTenPortionOfEnhancer[1]

  powerOfTenPortionsOfEnhancer := powerOfTenPortionOfEnhancer[0] * powerOfTenPortionOfEnhancer[1]
  otherPortionsOfEnhancer := otherPortionOfEnhancer[0] * otherPortionOfEnhancer[1]
         
PUB DisplayEnhancers

  Pst.Str(string(13, " LARGEST_PRACTICAL_MULTIPLIER = "))
  Pst.Dec(LARGEST_PRACTICAL_MULTIPLIER)
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[0] = "))
  Pst.Dec(integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[0] = "))
  Pst.Dec(otherPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[0] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, " integerMathEnhancer[1] = "))
  Pst.Dec(integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionOfEnhancer[1] = "))
  Pst.Dec(otherPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionOfEnhancer[1] = "))
  Pst.Dec(powerOfTenPortionOfEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, " powerOfTenPortionsOfEnhancer = "))
  Pst.Dec(powerOfTenPortionsOfEnhancer)
  Pst.ClearEnd
  Pst.Str(string(13, " otherPortionsOfEnhancer = "))
  Pst.Dec(otherPortionsOfEnhancer)
  Pst.ClearEnd

PUB ClearLines(lineToClear, lastLine)

  repeat
    Pst.Position(0, lineToClear++)
    Pst.ClearEnd 
  while lineToClear =< lastLine
  
PUB ReadableBin(localValue)

  Pst.char("%")
  Pst.bin(localValue >> 28, 4)
  Pst.char("_")
  Pst.bin(localValue >> 24, 4)
  Pst.char("_")
  Pst.bin(localValue >> 20, 4)
  Pst.char("_")
  Pst.bin(localValue >> 16, 4)
  Pst.char("_")
  Pst.bin(localValue >> 12, 4)
  Pst.char("_")
  Pst.bin(localValue >> 8, 4)
  Pst.char("_")
  Pst.bin(localValue >> 4, 4)
  Pst.char("_")
  Pst.bin(localValue, 4)

PUB DecPoint(value, denominator)
  if value < 0
    Pst.Char("-")
    -value
      
  if value => denominator
    result := value / denominator
    Pst.Dec(result)
    value //= denominator     
  else    
    Pst.Char("0")
  Pst.Char(".")  
  repeat while denominator > 1
    denominator /= 10
    if value => denominator
      result := value / denominator
      Pst.Dec(result)
      value //= denominator
    else
      Pst.Char("0")
      
PUB Pad(value, digits) | negFlag
'' This method will pad a number with spaces.
'' The number will take up digits + 1 positions.
'' The extra position is used for a negative
'' sign in case of a negative number.

  if value < 0
    negFlag := 1
    -value
  else
    negFlag := 0
    
  result := 1
  repeat digits - 1
    result *= 10
    
  repeat digits
    if value < result 
      Pst.Char(" ")
      result /= 10
    else
      if negFlag
        Pst.Char("-")
      else
        Pst.Char(" ")
      Pst.Dec(value)
      quit

  ifnot result
    Pst.Char("0")
           
PUB StepByStepDebug(localIndex)
'' Used to catch 32-bit overflow.

  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR)
  Pst.ClearEnd
  Pst.Str(string(13, "clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0] = "))
  Pst.Dec(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0])
  Pst.ClearEnd
  Pst.Str(string(13, "(clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result] = "))
  Pst.Dec((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex])
  Pst.ClearEnd
  Pst.Str(string(13, "((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60 = "))
  Pst.Dec(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60)
  Pst.ClearEnd
  Pst.Str(string(13, "(((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1] = "))
  Pst.Dec((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1])
  Pst.ClearEnd
  Pst.Str(string(13, "(((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * integerMathEnhancer[0]) "))
  Pst.Str(string("/ frozenTransitionTime[result]) * 60) * integerMathEnhancer[1]) / "))
  Pst.Str(string("Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer = "))
  Pst.Dec((((((clkfreq * Header#TRANS_PER_REV_DENOMINATOR * {
      } integerMathEnhancer[0]) / frozenTransitionTime[localIndex]) * {
      } 60) * integerMathEnhancer[1]) / Header#TRANS_PER_REV_NUMERATOR) / otherPortionsOfEnhancer)
  Pst.ClearEnd

PRI ExtraDebug
'' Not used.

  {Pst.Str(string(13, "    linearSpeed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(linearSpeed[result])
  Pst.ClearEnd
  Pst.Str(string(13, " based on wheel diameter of "))
  Pst.Dec(Header#DIAMETER_OF_WHEEL) 
  Pst.Str(Header.GetLinearUnits)
  Pst.ClearEnd      }
   
  Pst.Str(string(13, "previousTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(previousEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenTransitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenTransitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "frozenLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "previousEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(frozenEncoderCount[result])
  Pst.ClearEnd
  Pst.Str(string(13, "legacyLongsNotUsed["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(legacyLongsNotUsed[result])
  Pst.ClearEnd
  Pst.Str(string(13, "transitionTime["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(transitionTime[result])
  Pst.ClearEnd
  Pst.Str(string(13, "timeOfLastPulse["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(timeOfLastPulse[result])
  Pst.ClearEnd
  Pst.Str(string(13, "totalEncoderCount["))
  Pst.Dec(result)
  Pst.Str(string("] = "))
  Pst.Dec(totalEncoderCount[result])
  Pst.ClearEnd

DAT

astrixRow               byte "*"[ASTRIX_IN_ROW], 0
dashRow                 byte "-"[LINE_SIZE], 0

characterInputError     byte "Not a Valid Input Character", 0
parameterNotYetError    byte "Parameter Not Presently Adjustable", 0
speedInputError         byte "Not a Valid Speed", 0
                   